	printf("Begin of second part\n");

	/*
	 * Lorem ipsum dolor sit amet, consectetuer sadipscing elitr,
	 * sed diam nonumy eirmod tempor invidunt ut labore et dolore
	 * magna aliquyam erat, sed diam voluptua. At vero eos et
	 * accusam et justo duo dolores et ea rebum. Stet clita kasd
	 * gubergren, no sea takimata sanctus est Lorem ipsum dolor
	 * sit amet.
	 *
	 * Lorem ipsum dolor sit amet, consectetuer sadipscing elitr,
	 * sed diam nonumy eirmod tempor invidunt ut labore et dolore
	 * magna aliquyam erat, sed diam voluptua. At vero eos et
	 * accusam et justo duo dolores et ea rebum. Stet clita kasd
	 * gubergren, no sea takimata sanctus est Lorem ipsum dolor
	 * sit amet.
	 *
	 * Lorem ipsum dolor sit amet, consectetuer sadipscing elitr,
	 * sed diam nonumy eirmod tempor invidunt ut labore et dolore
	 * magna aliquyam erat, sed diam voluptua. At vero eos et
	 * accusam et justo duo dolores et ea rebum. Stet clita kasd
	 * gubergren, no sea takimata sanctus est Lorem ipsum dolor
	 * sit amet.
	 *
	 * Lorem ipsum dolor sit amet, consectetuer sadipscing elitr,
	 * sed diam nonumy eirmod tempor invidunt ut labore et dolore
	 * magna aliquyam erat, sed diam voluptua. At vero eos et
	 * accusam et justo duo dolores et ea rebum. Stet clita kasd
	 * gubergren, no sea takimata sanctus est Lorem ipsum dolor
	 * sit amet.
	 *
	 */

	return 0;
}	// End of second part
