1 VERSIONINFO
FILEVERSION     MAJOR,MINOR,MICRO,PATCHLEVEL
PRODUCTVERSION  MAJOR,MINOR,MICRO,PATCHLEVEL
BEGIN
  BLOCK "StringFileInfo"
  BEGIN
    BLOCK "040904b0"   /* LANG_ENGLISH/SUBLANG_ENGLISH_US, Unicode CP */
    BEGIN
      VALUE "CompanyName", "The Git Development Community\0"
      VALUE "FileDescription", "Git for Windows\0"
      VALUE "InternalName", "git\0"
      VALUE "OriginalFilename", "git.exe\0"
      VALUE "ProductName", "Git\0"
      VALUE "ProductVersion", GIT_VERSION "\0"
    END
  END

  BLOCK "VarFileInfo"
  BEGIN
    VALUE "Translation", 0x409, 1200
  END
END
